/*
    daytimed.c
    Version 1.0
    Copyright 1997 by Richard Dawe

    Simple daytime service provider

    ---

    Richard Dawe's contact details:

    E-mail: rd5718@bristol.ac.uk
    Web:    http://irix.bris.ac.uk/~rd5718/

    ---

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/* Include files */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/socket.h>

/* --------
   - main -
   -------- */

int main (void)
{
    int                sock, sock_peer, slen;
    struct protoent    *proto;
    struct servent     *serv;
    struct hostent     *host;
    struct sockaddr_in sin, sin_peer;
    time_t             time_now;
    char               time_now_str[256];

    /* Copyright message */
    printf("daytimed Version 1.0, Copyright 1997 by Richard Dawe\n");
    printf("This program is distributed under the terms of the GNU General Public\nLicense. Please see the file LICENSE for details.\n");

    /* Get protocol details & create socket */
    proto = getprotobyname("tcp");
    sock = socket(AF_INET, SOCK_STREAM, proto->p_proto);
    if (sock == -1) exit(0);

    /* Get service details, bind socket to any incoming internet address */
    serv = getservbyname("daytime", "tcp");

    memset(&sin, 0, sizeof(sin));
    sin.sin_family = AF_INET;
    sin.sin_addr.s_addr = INADDR_ANY;
    sin.sin_port = serv->s_port;

    if (bind(sock, (struct sockaddr *) &sin, sizeof(struct sockaddr)) == -1)
        exit(1);

    /* Queue upto 5 requests */
    listen(sock, 5);

    /* Accept any incoming requests until the program ends */
    slen = sizeof(struct sockaddr_in);

    while ((sock_peer = accept(sock, (struct sockaddr *) &sin_peer, &slen)) != -1)
    {
        /* Get the local time and return it to the client */
        time(&time_now);
        sprintf(time_now_str, "%s", asctime(localtime(&time_now)));
        send(sock_peer, time_now_str, (strlen(time_now_str) + 1), 0);
        close(sock_peer);
    }

    /* OK */
    return(0);
}
